import 'dart:math' as math;

class Region {
  final math.Rectangle<int> rect;
  final Set<(int, int)> points;

  bool get isRectangle => points.length == rect.width * rect.height;

  Region(this.rect, this.points);

  static Region fromNonEmptySet(Set<(int, int)> s) {
    assert(s.isNotEmpty);
    int xMin = s.map<int>((xy) => xy.$1).reduce(math.min);
    int yMin = s.map<int>((xy) => xy.$2).reduce(math.min);
    int xMax = s.map<int>((xy) => xy.$1).reduce(math.max);
    int yMax = s.map<int>((xy) => xy.$2).reduce(math.max);
    var rect = math.Rectangle.fromPoints(
        math.Point(xMin, yMin), math.Point(xMax + 1, yMax + 1));
    return Region(rect, s);
  }

  @override
  String toString() {
    return "Region($rect, $points)";
  }
}

List<Region> regionalize(
    math.Rectangle<int> rect, bool Function(int, int) isAccessible) {
  int nextRegion = 0;
  Map<(int, int), int> regions = {};

  int floodfill(int x, int y) {
    int workDone = 0;
    if (!rect.containsPoint(math.Point(x, y))) {
      return workDone;
    }
    if (regions[(x, y)] != null) {
      return workDone;
    }
    if (!isAccessible(x, y)) {
      return workDone;
    }

    regions[(x, y)] = nextRegion;
    workDone += 1;
    workDone += floodfill(x - 1, y);
    workDone += floodfill(x + 1, y);
    workDone += floodfill(x, y - 1);
    workDone += floodfill(x, y + 1);
    return workDone;
  }

  // TODO: This can be done more efficiently with a union/find data structure
  // But this is an easy implementation to understand
  for (var y = rect.top; y < rect.bottom; y++) {
    for (var x = rect.left; x < rect.right; x++) {
      if (regions[(x, y)] == null) {
        if (floodfill(x, y) > 0) {
          nextRegion += 1;
        }
      }
    }
  }
  return _toExplicit(regions, nextRegion);
}

List<Region> _toExplicit(Map<(int, int), int> pointRegions, int nRegions) {
  List<Set<(int, int)>> regionPoints = [for (var i = 0; i < nRegions; i++) {}];
  for (var MapEntry(key: (x, y), value: id_) in pointRegions.entries) {
    regionPoints[id_].add((x, y));
  }

  return [for (var s in regionPoints) Region.fromNonEmptySet(s)];
}
